
//		RENCASE command, and related functions:


#ifndef RENCASE_VER
#define RENCASE_VER				1.0.2.0


#define RC_BASE_LOWER			0x0001
#define RC_BASE_UPPER			0x0002
#define RC_BASE_WORD			0x0003
#define RC_BASE_BITS			0x0003
#define RC_EXT_LOWER			0x0004
#define RC_EXT_UPPER			0x0008
#define RC_EXT_WORD				0x000c
#define RC_EXT_BITS				0x000c
#define RC_FILENAME_BITS		0x000f
#define RC_QUIET				0x0100
#define RC_COMPLAIN				0x0200
#define RC_NOT_REALLY			0x1000

#define RC_MODE_LOWER			0x0001
#define RC_MODE_UPPER			0x0002
#define RC_MODE_WORD			0x0003




const wchar_t RenCaseHelpText[] = L"Change the case of filenames.\n\
\n\
RENCASE /A: /L /N /S /U /W /X: file....\r\n\
\n\
\t/A:\tattributes\n\
\t/L\trename to lowercase\n\
\t/N\tnot really\n\
\t/S\tsearch in subdirectories\n\
\t/U\trename to uppercase\n\
\t/W\trename to word case\n\
\t/X:\trename extension: U L or W  \n\
\n\
You must include least one filename.  If you do not specify any  \n\
of /L /U /W or /X:, the default is /L.\n";



int RenCaseFile( LPCTSTR Filename, int Flags, void *MoreArgs )
{
	if ( !Filename )
		return -1;
	else if ( Filename[0] == '\0' )
		return -1;


	if ( ( Flags & RC_FILENAME_BITS ) == 0 )
		return 1;


	wchar_t Path[MAX_FILENAME_LEN]			= L"";
	wchar_t Name[MAX_FILENAME_LEN]			= L"";
	wchar_t NewName[MAX_FILENAME_LEN]		= L"";
	wchar_t ErrorText[SHORT_BUF_LEN]		= L"";
	LPTSTR Ext								= NULL;
	int Mode								= Flags & RC_BASE_BITS;
	BOOL Start								= TRUE;
	wchar_t ch								= '\0';
	BOOL Quiet								= Flags & RC_QUIET;
	BOOL Complain							= Flags & RC_COMPLAIN;


	if ( !QueryIsFileOrDirectory( (LPTSTR) Filename ) )
		return ERROR_FILE_NOT_FOUND;


	FH_PathPart( Filename, Path );
	FH_FilenamePart( Filename, Name );
	Ext = FH_ExtensionPart( Name, NULL );


	for ( unsigned int i = 0; Name[i] ; i++ ) {
		ch = Name[i];

		switch( Mode ) {

		case RC_MODE_LOWER :
			ch = towlower( ch );
			break;

		case RC_MODE_UPPER :
			ch = towupper( ch );
			break;

		case RC_MODE_WORD :
			if ( Start )
				ch = towupper( ch );
			else
				ch = towlower( ch );

			if ( ch == '\'' )
				;
			else if ( iswalpha( ch ) )
				Start = FALSE;
			else
				Start = TRUE;
			
			break;
		}

		if ( Ext )
			if ( ( Name + i ) == Ext )
				Mode = ( Flags & RC_EXT_BITS ) >> 2;

		NewName[i] = ch;
		NewName[i + 1] = '\0';
	}


	if ( !wcscmp( Name, NewName ) ) {
		if ( !Quiet )
			Printf( L"Skipping: %s\r\n", Filename );

		return 0;
	}


	if ( Flags & RC_NOT_REALLY ) {
		Printf( L"%s \u2014> %s\r\n", Filename, NewName );
		return 0;
	}
	else if ( !MoveFile( Filename, NewName ) ) {
		if ( Complain ) {
			GetErrorText( GetLastError(), ErrorText );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"%s\r\n", ErrorText );
			Qprintf( ERRHANDLE, L"   %s\r\n", Filename );
		}

		return 2;
	}
	
	if ( !Quiet )
		Printf( L"%s \u2014> %s\r\n", Filename, NewName );

	return 0;
}



DLLExports int rencase( LPTSTR inString )
{
	if ( !inString )
		return 1;


	int Flags					= RC_COMPLAIN;
	int Recurse					= FH_FLAG_INCLUDE_LISTS;
	unsigned int MinDepth		= 0;
	unsigned int MaxDepth		= UINT_MAX;
	DWORD AttribMask			= FILE_ATTRIBUTE_DIRECTORY;
	DWORD AttribValues			= 0;
	void *FirstRange			= NULL;

	wchar_t Wildspec[MAX_FILENAME_LEN]		= L"";

	int NumFilespecs			= 0;
	int rv						= 1;
	int t						= 0;



	ParseArgs( inString, PARSE_BREAK_SPACES | PARSE_EXCL_RANGE_KLUDGE );


	for ( int i = 0; i < numargs; i++ ) {
		if ( arglen[i] ) {
			if ( !wcsncmp( arg[i], L"/?", 2 ) ) {
				ShowCmdHelp( RenCaseHelpText, ( arg[i][2] == '?' ), __FILE__, MACRO_STR( RENCASE_VER ) );
				rv = 0;
				goto CleanupAndExit;
			}
			else if ( !_wcsicmp( arg[i], L"/N" ) )
				Flags |= RC_NOT_REALLY;
			else if ( t = ParseCommonOption( arg[i], TRUE, &AttribMask, &AttribValues, &Recurse, &MinDepth, &MaxDepth, NULL, &FirstRange ) ) {
				if ( t < 0 )
					goto CleanupAndExit;
			}
			else if ( !_wcsicmp( arg[i], L"/E" ) )
				Flags &= ~RC_COMPLAIN;
			else if ( !_wcsicmp( arg[i], L"/L" ) )
				Flags = ( Flags & ~RC_BASE_BITS ) | RC_BASE_LOWER;
			else if ( !_wcsicmp( arg[i], L"/U" ) )
				Flags = ( Flags & ~RC_BASE_BITS ) | RC_BASE_UPPER;
			else if ( !_wcsicmp( arg[i], L"/W" ) )
				Flags = ( Flags & ~RC_BASE_BITS ) | RC_BASE_WORD;
			else if ( !_wcsnicmp( arg[i], L"/X", 2 ) ) {
				int j = 2;
				if ( IsColonOrEquals( arg[i][j] ) )
					j++;

				switch ( towupper( arg[i][j] ) ) {

				case 'L' :
					Flags = ( Flags & ~RC_EXT_BITS ) | RC_EXT_LOWER;
					break;

				case 'U' :
					Flags = ( Flags & ~RC_EXT_BITS ) | RC_EXT_UPPER;
					break;

				case 'W' :
					Flags = ( Flags & ~RC_EXT_BITS ) | RC_EXT_WORD;
					break;

				default :
					DisplayErrorHeader();
					Qprintf( ERRHANDLE, L"Unknown suboption: /X:%c\r\n", towupper( arg[i][j] ) );
					goto CleanupAndExit;
				}
			}
			else if ( !_wcsicmp( arg[i], L"/Q" ) )
				Flags |= RC_QUIET;
			else if ( arg[i][0] == '/' ) {
				_wcsupr_s( arg[1], MAX_ARG_LEN );
				Qprintf( ERRHANDLE, L"Unknown option: \"%s\"\r\n", arg[i] );
				goto CleanupAndExit;
			}
			else
				NumFilespecs++;
		}
	}


	if ( !NumFilespecs ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Missing filename\r\n" );
		goto CleanupAndExit;
	}

	if ( ( Flags & RC_FILENAME_BITS ) == 0 )
		Flags |= RC_BASE_LOWER;


	rv = 0;


	for ( int i = 0; i < numargs; i++ ) {
		if ( arglen[i] ) {
			if ( arg[i][0] != '/' ) {
				FH_Truename( arg[i], Wildspec, MAX_FILENAME_LEN, TRUENAME_FLAG_HOME_PATHS | TRUENAME_FLAG_INCLUDE_LISTS );
				if ( HandleMatchingFiles( Wildspec, Flags, NULL, RenCaseFile, Recurse, AttribMask, AttribValues, FirstRange, MinDepth, MaxDepth ) )
					rv = 2;
			}
		}
	}


CleanupAndExit:

	FH_FreeLinkedList( &FirstRange );
	return rv;
}


#endif	//	RENCASE_VER


/*

	1.0.0.0		2018-05-10

	1.0.1.0		2022-03-01
				Reworked to use a newer version of FileHandler.cpp.  No-Complain option is now /E,
				not /NE.  Help text is now displayed via ShowCmdHelp().

	1.0.2.0		2023-08-22
				RenCaseFile()'s filename arg is now defined as a constant.

*/
